
/*
     ______     __                     ___     __
    /\  _  \   /\ \                   /\_ \   /\_\
    \ \ \_\ \  \ \ \      __     ___  \//\ \  \/_/_
     \ \  __ \ `/__ \   /'__`\  / __`\  \ \ \   /\ \
      \ \ \/\ \/\\_\ \_/\ \_\ \/\ \_\ \_ \_\ \_ \_\ \_
       \ \_\ \_\ \_____\ \___\ \ \______\/\____\/\____\
        \/_/\/_/\/_____/\/___L\ \/______/\/____/\/____/
                          /\____/
                          \_/__/




                  An advanced game library

                 (c) Stephane APIOU 2003


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



*/

//! definition of all adgali features associated with ios

/*! \file
 * \ingroup ios
 * this file defines all adgali features associated with ios
 * and must be include when you use adgali
 * this file is included by agl.h
 */


#ifndef _adgali_ios_h
#define _adgali_ios_h

#include <stdio.h>

//@}

struct _agl_iosio;

//! defines an io stream

/*! This structure defines io streams for adgali
 * all the members are private. Don't use them directly
 */
typedef struct agl_ios
{
	agl_handle handle;
	struct _agl_iosio *pio;
	char *filename;		//!< the ( if defined ) name of the file
	void *priv;
} agl_ios;

typedef struct _agl_iosio
{
	char *desc;		//!< descriptor of input/output openened
	int (*is) (agl_module *, char *, agl_ios *);
	agl_ios *(*open) (agl_module *, char *, agl_ios *);
	long (*seek) (agl_ios *, long, int);	//!< the seek function
	long (*read) (agl_ios *, void *, long, long, void *);	//!< the read function
	long (*write) (agl_ios *, void *, long, long, void *);	//!< the write function
	int (*close) (agl_ios *);	//!< the close function
} agl_iosio;


extern agl_ios *agl_ios_open(agl_handle handle, char *address);

//! seek into the stream

/*! this function seek in a stream
 * \param ios an agl_ios pointer which defines the stream
 * \param offset an offset in then stream. The value could be positive or negative
 * \param whence indicates the type of offset. see also seek(2) function.
 * - SEEK_SET: force offset from the beginning of the stream
 * - SEEK_CUR: force offset from the current position of the stream
 * - SEEK_END: force offset from the end of the stream
 * \return the offset from the beginning of the stream
 * \note the position of the stream could not be negative and greater than the
 * stream size.
 */

#define agl_ios_seek(ios, offset, whence) ((ios)->pio->seek(ios, offset, whence))

//! tell the position in the stream

/*! this function tell the position of the read/write pointer in the stream
 * \param ios an agl_ios pointer which defines the stream
 * \return the offset from the beginning of the stream
 */

#define agl_ios_tell(ios) ((ios)->pio->seek(ios, 0, SEEK_CUR))

//! read the stream

/*! this function read the stream into a buffer
 * \param ios an agl_ios pointer which defines the stream
 * \param buf point to the read buffer. The size of this buffer should be big
 * enough to contain \a size * \a maxnum characters
 * \param size this is the size of one item
 * \param maxnum this the maximum number of item to read
 * \param supp this parameter is reserved for future use (socket for example)
 * \return the number of items read
 */

#define agl_ios_read(ios, buf, size, maxnum, supp) ((ios)->pio->read(ios, buf, size, maxnum ,supp))

//! write into the stream

/*! this function write a buffer into the stream
 * \param ios an agl_ios pointer which defines the stream
 * \param buf point to the write buffer. The size of this buffer should be big
 * enough to contain \a size * \a maxnum characters
 * \param size this is the size of one item
 * \param maxnum this the maximum number of item to read
 * \param supp this parameter is reserved for future use (socket for example)
 * \return the number of items written
 */

#define agl_ios_write(ios, buf, size, maxnum, supp) ((ios)->pio->write(ios, buf, size, maxnum ,supp))

//! close the stream

/*! this function close the stream
 * \param ios an agl_ios pointer which defines the stream
 * \return nothing
 */


#define agl_ios_close(ios) ((ios)->pio->close(ios))


#endif /* _agl_ios_h */
